'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');
var compositionApi = require('@vue/composition-api');
var vue = require('vue');

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

var script = compositionApi.defineComponent({
    name: 'FileUploader',
    // 组件属性定义
    props: {
        action: {
            type: String,
            required: true
        },
        multiple: {
            type: Boolean,
            default: false
        },
        accept: {
            type: String,
            default: ''
        },
        maxSize: {
            type: Number,
            default: 0
        },
        maxCount: {
            type: Number,
            default: 0
        },
        headers: {
            type: Object,
            default: () => ({})
        },
        data: {
            type: Object,
            default: () => ({})
        },
        autoUpload: {
            type: Boolean,
            default: true
        },
        fileList: {
            type: Array,
            default: () => []
        },
        disabled: {
            type: Boolean,
            default: false
        },
        drag: {
            type: Boolean,
            default: false
        }
    },
    // 组件事件定义
    emits: ['change', 'success', 'error', 'progress', 'exceed', 'update:fileList'],
    setup(props, { emit, expose }) {
        // 文件输入框引用
        const fileInput = compositionApi.ref(null);
        // 文件列表状态
        const fileList = compositionApi.ref(props.fileList || []);
        // 存储待上传的文件
        const pendingFiles = compositionApi.ref(new Map());
        // 拖拽状态
        const isDragging = compositionApi.ref(false);
        // 使用 watch 监听 props.fileList 的变化
        compositionApi.watch(() => props.fileList, (newValue) => {
            fileList.value = newValue || [];
        });
        /**
         * 处理点击上传
         * 触发文件选择框
         */
        const handleClick = () => {
            var _a;
            (_a = fileInput.value) === null || _a === void 0 ? void 0 : _a.click();
        };
        /**
         * 处理文件选择
         * 当用户通过文件选择框选择文件时触发
         */
        const handleChange = (event) => {
            var _a;
            const target = event.target;
            if (!((_a = target.files) === null || _a === void 0 ? void 0 : _a.length))
                return;
            const files = Array.from(target.files);
            handleFiles(files);
            target.value = ''; // 重置input，允许选择相同文件
        };
        /**
         * 处理文件拖拽
         * 当用户拖拽文件到上传区域时触发
         */
        const handleDrop = (event) => {
            var _a;
            const files = Array.from(((_a = event.dataTransfer) === null || _a === void 0 ? void 0 : _a.files) || []);
            handleFiles(files);
        };
        /**
         * 处理文件上传
         * 验证文件并添加到上传列表
         */
        const handleFiles = (files) => {
            // 检查文件数量限制
            if (props.maxCount && fileList.value.length + files.length > props.maxCount) {
                emit('exceed', files);
                return;
            }
            files.forEach(file => {
                // 检查文件大小限制
                if (props.maxSize && file.size > props.maxSize) {
                    const errorFile = {
                        uid: Date.now() + Math.random().toString(36).slice(2),
                        name: file.name,
                        size: file.size,
                        type: file.type,
                        status: 'error',
                        error: new Error('文件大小超出限制')
                    };
                    emit('error', errorFile.error, errorFile);
                    return;
                }
                // 创建上传文件对象
                const uploadFile = {
                    uid: Date.now() + Math.random().toString(36).slice(2),
                    name: file.name,
                    size: file.size,
                    type: file.type,
                    status: 'ready',
                    percentage: 0
                };
                // 添加到文件列表
                fileList.value.push(uploadFile);
                // 存储原始文件对象，用于后续上传
                pendingFiles.value.set(uploadFile.uid, file);
                emit('update:fileList', fileList.value);
                emit('change', { file: uploadFile, fileList: fileList.value });
                // 自动上传
                if (props.autoUpload) {
                    upload(file, uploadFile);
                }
            });
        };
        /**
         * 处理拖拽进入
         */
        const handleDragover = () => {
            isDragging.value = true;
        };
        /**
         * 处理拖拽离开
         */
        const handleDragleave = () => {
            isDragging.value = false;
        };
        /**
         * 上传文件到服务器
         * 使用 fetch API 发送文件，支持进度显示
         */
        const upload = (file, uploadFile) => __awaiter(this, void 0, void 0, function* () {
            const formData = new FormData();
            formData.append('file', file);
            // 添加额外参数
            if (props.data) {
                Object.entries(props.data || {}).forEach(([key, value]) => {
                    formData.append(key, value);
                });
            }
            uploadFile.status = 'uploading';
            uploadFile.percentage = 0;
            try {
                const xhr = new XMLHttpRequest();
                // 监听上传进度
                xhr.upload.onprogress = (e) => {
                    if (e.lengthComputable) {
                        uploadFile.percentage = Math.round((e.loaded * 100) / e.total);
                        emit('update:fileList', fileList.value);
                    }
                };
                // 创建 Promise 包装 XHR
                const response = yield new Promise((resolve, reject) => {
                    xhr.onload = () => {
                        if (xhr.status >= 200 && xhr.status < 300) {
                            try {
                                resolve(JSON.parse(xhr.responseText));
                            }
                            catch (e) {
                                reject(new Error('解析响应数据失败'));
                            }
                        }
                        else {
                            reject(new Error(`上传失败: ${xhr.status}`));
                        }
                    };
                    xhr.onerror = () => reject(new Error('网络错误'));
                    // 设置请求头
                    const headers = {};
                    if (props.headers) {
                        Object.entries(props.headers || {}).forEach(([key, value]) => {
                            headers[key] = value;
                        });
                    }
                    // 发送请求
                    xhr.open('POST', props.action);
                    Object.entries(headers).forEach(([key, value]) => {
                        xhr.setRequestHeader(key, value);
                    });
                    xhr.send(formData);
                });
                uploadFile.status = 'success';
                uploadFile.response = response;
                uploadFile.url = response.url;
                emit('success', uploadFile, fileList.value);
            }
            catch (error) {
                uploadFile.status = 'error';
                uploadFile.error = error;
                emit('error', error, uploadFile);
            }
            emit('update:fileList', fileList.value);
        });
        /**
         * 从列表中移除文件
         */
        const removeFile = (file) => {
            const index = fileList.value.indexOf(file);
            if (index > -1) {
                fileList.value.splice(index, 1);
                emit('update:fileList', fileList.value);
            }
        };
        /**
         * 格式化文件大小
         * 将字节数转换为人类可读的格式
         */
        const formatSize = (bytes) => {
            if (bytes === 0)
                return '0 B';
            const k = 1024;
            const sizes = ['B', 'KB', 'MB', 'GB', 'TB'];
            const i = Math.floor(Math.log(bytes) / Math.log(k));
            return `${(bytes / Math.pow(k, i)).toFixed(2)} ${sizes[i]}`;
        };
        /**
         * 手动上传文件
         * 上传所有状态为 ready 的文件
         */
        const submit = () => {
            fileList.value.forEach((uploadFile) => {
                if (uploadFile.status === 'ready') {
                    const file = pendingFiles.value.get(uploadFile.uid);
                    if (file) {
                        upload(file, uploadFile);
                    }
                }
            });
        };
        // 暴露方法给父组件
        expose({
            submit,
            handleClick,
            fileInput
        });
        return {
            fileInput,
            fileList,
            handleClick,
            handleChange,
            handleDrop,
            handleDragover,
            handleDragleave,
            removeFile,
            formatSize,
            submit,
            isDragging
        };
    }
});

const _hoisted_1 = { class: "vue-file-uploader" };
const _hoisted_2 = {
  key: 0,
  class: "uploader-list"
};
const _hoisted_3 = { class: "file-status-indicator" };
const _hoisted_4 = { class: "file-info" };
const _hoisted_5 = ["title"];
const _hoisted_6 = { class: "file-size" };
const _hoisted_7 = {
  key: 0,
  class: "file-status"
};
const _hoisted_8 = {
  key: 1,
  class: "file-status-text"
};
const _hoisted_9 = {
  key: 0,
  class: "success"
};
const _hoisted_10 = ["title"];
const _hoisted_11 = ["onClick", "title"];
const _hoisted_12 = ["accept", "multiple"];

function render(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", _hoisted_1, [
    vue.createCommentVNode(" 上传触发区域：点击或拖拽上传 "),
    vue.createElementVNode("div", {
      class: vue.normalizeClass(["uploader-trigger", { 'dragging': _ctx.isDragging }]),
      onDrop: _cache[1] || (_cache[1] = vue.withModifiers((...args) => (_ctx.handleDrop && _ctx.handleDrop(...args)), ["prevent"])),
      onDragover: _cache[2] || (_cache[2] = vue.withModifiers((...args) => (_ctx.handleDragover && _ctx.handleDragover(...args)), ["prevent"])),
      onDragleave: _cache[3] || (_cache[3] = vue.withModifiers((...args) => (_ctx.handleDragleave && _ctx.handleDragleave(...args)), ["prevent"]))
    }, [
      vue.createCommentVNode(" 默认上传区域，可通过插槽自定义 "),
      vue.renderSlot(_ctx.$slots, "default", {}, () => [
        vue.createElementVNode("div", {
          class: "uploader-placeholder",
          onClick: _cache[0] || (_cache[0] = (...args) => (_ctx.handleClick && _ctx.handleClick(...args)))
        }, _cache[5] || (_cache[5] = [
          vue.createElementVNode("i", { class: "uploader-icon" }, "+", -1 /* HOISTED */),
          vue.createElementVNode("div", { class: "uploader-text" }, "点击或拖拽文件上传", -1 /* HOISTED */)
        ]))
      ])
    ], 34 /* CLASS, NEED_HYDRATION */),
    vue.createCommentVNode(" 文件列表展示区域 "),
    (_ctx.fileList.length)
      ? (vue.openBlock(), vue.createElementBlock("div", _hoisted_2, [
          (vue.openBlock(true), vue.createElementBlock(vue.Fragment, null, vue.renderList(_ctx.fileList, (file) => {
            return (vue.openBlock(), vue.createElementBlock("div", {
              key: file.uid,
              class: "uploader-file"
            }, [
              vue.createCommentVNode(" 文件状态指示器 "),
              vue.createElementVNode("div", _hoisted_3, [
                vue.createElementVNode("span", {
                  class: vue.normalizeClass(["icon", file.status])
                }, [
                  (file.status === 'uploading')
                    ? (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 0 }, [
                        vue.createTextVNode("↻")
                      ], 64 /* STABLE_FRAGMENT */))
                    : (file.status === 'success')
                      ? (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 1 }, [
                          vue.createTextVNode("✓")
                        ], 64 /* STABLE_FRAGMENT */))
                      : (file.status === 'error')
                        ? (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 2 }, [
                            vue.createTextVNode("✕")
                          ], 64 /* STABLE_FRAGMENT */))
                        : (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 3 }, [
                            vue.createTextVNode("•")
                          ], 64 /* STABLE_FRAGMENT */))
                ], 2 /* CLASS */)
              ]),
              vue.createCommentVNode(" 文件信息展示 "),
              vue.createElementVNode("div", _hoisted_4, [
                vue.createElementVNode("span", {
                  class: "file-name",
                  title: file.name
                }, vue.toDisplayString(file.name), 9 /* TEXT, PROPS */, _hoisted_5),
                vue.createElementVNode("span", _hoisted_6, vue.toDisplayString(_ctx.formatSize(file.size)), 1 /* TEXT */)
              ]),
              vue.createCommentVNode(" 上传进度/状态 "),
              (file.status === 'uploading')
                ? (vue.openBlock(), vue.createElementBlock("div", _hoisted_7, [
                    vue.createElementVNode("div", {
                      class: "progress-bar",
                      style: vue.normalizeStyle({ width: `${file.percentage}%` })
                    }, null, 4 /* STYLE */)
                  ]))
                : (vue.openBlock(), vue.createElementBlock("div", _hoisted_8, [
                    (file.status === 'success')
                      ? (vue.openBlock(), vue.createElementBlock("span", _hoisted_9, "完成"))
                      : vue.createCommentVNode("v-if", true),
                    (file.status === 'error')
                      ? (vue.openBlock(), vue.createElementBlock("span", {
                          key: 1,
                          class: "error",
                          title: file.error?.message
                        }, " 失败 ", 8 /* PROPS */, _hoisted_10))
                      : vue.createCommentVNode("v-if", true)
                  ])),
              vue.createCommentVNode(" 删除按钮 "),
              vue.createElementVNode("button", {
                class: "delete-btn",
                onClick: $event => (_ctx.removeFile(file)),
                title: '删除 ' + file.name
              }, " ✕ ", 8 /* PROPS */, _hoisted_11)
            ]))
          }), 128 /* KEYED_FRAGMENT */))
        ]))
      : vue.createCommentVNode("v-if", true),
    vue.createCommentVNode(" 隐藏的文件输入框 "),
    vue.createElementVNode("input", {
      ref: "fileInput",
      type: "file",
      accept: _ctx.accept,
      multiple: _ctx.multiple,
      style: {"display":"none"},
      onChange: _cache[4] || (_cache[4] = (...args) => (_ctx.handleChange && _ctx.handleChange(...args)))
    }, null, 40 /* PROPS, NEED_HYDRATION */, _hoisted_12)
  ]))
}

script.render = render;
script.__file = "src/components/FileUploader.vue";

/**
 * Vue File Uploader 组件入口文件
 * 支持 Vue 2.x 和 Vue 3.x
 */
// 如果是 Vue 2 环境，确保已安装 composition-api
if (vueDemi.isVue2) {
    try {
        require('@vue/composition-api');
    }
    catch (e) {
        console.error('[vue-file-uploader] When using Vue 2, @vue/composition-api is required.');
    }
}
/**
 * Vue 插件安装函数
 * 用于全局注册组件
 * @param app Vue 应用实例
 */
const install = (app) => {
    app.component('FileUploader', script);
};
// 导出 Vue 插件
var index = {
    install,
    FileUploader: script
};
if (typeof window !== 'undefined' && window.Vue) {
    install(window.Vue);
}

exports.FileUploader = script;
exports["default"] = index;
//# sourceMappingURL=vue-file-uploader.common.js.map
